/***********************************************************************************************************************
 *
 * blueArgyle - a Java UI for Argyll
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://blueargyle.java.net
 * SCM: https://bitbucket.org/tidalwave/blueargyle-src
 *
 **********************************************************************************************************************/
package it.tidalwave.colorimetry;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import it.tidalwave.util.NotFoundException;
import lombok.EqualsAndHashCode;
import lombok.Getter;

/***********************************************************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Immutable @Getter @EqualsAndHashCode
public class ColorPoints
  {
    private final List<ColorPoint> colorPoints = new ArrayList<ColorPoint>();
    
    public ColorPoints (final @Nonnull ColorPoint ... colorPoints)
      {
        this.colorPoints.addAll(Arrays.asList(colorPoints));
      }
    
    @Nonnull
    public <T extends ColorPoint> T find (final @Nonnull Class<T> clazz)
      throws NotFoundException
      {
        for (final ColorPoint colorPoint : colorPoints)
          {
            if (colorPoint.getClass().equals(clazz))
              {
                return clazz.cast(colorPoint);  
              }
          }
        
        throw new NotFoundException("No Color with space: " + clazz);
      }
    
    @Override @Nonnull
    public String toString()
      {
        return colorPoints.toString();  
      }
  }
